package com.github.johnnysc.holybibleapp.presentation.main

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.RecyclerView
import com.github.johnnysc.holybibleapp.R
import com.github.johnnysc.holybibleapp.core.Matcher

/**
 * @author Asatryan on 13.07.2021
 **/
abstract class BaseFragment<T : BaseViewModel> : Fragment(), Matcher<String> {

    protected lateinit var viewModel: T
    protected var recyclerView: RecyclerView? = null//todo viewbinding

    protected abstract fun viewModelClass(): Class<T>
    protected open fun layoutResId() = R.layout.fragment_main
    protected open fun showBackIcon() = true

    override fun matches(arg: String) = name() == arg
    fun name(): String = javaClass.simpleName

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        viewModel = (requireActivity() as MainActivity).getViewModel(viewModelClass(), this)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(layoutResId(), container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        (requireActivity() as AppCompatActivity).supportActionBar?.setDisplayHomeAsUpEnabled(
            showBackIcon()
        )
        updateTitle(viewModel.getTitle())
        recyclerView = view.findViewById(R.id.recyclerView)
        recyclerView?.addItemDecoration(
            DividerItemDecoration(
                requireContext(),
                DividerItemDecoration.VERTICAL
            )
        )
    }

    protected fun updateTitle(title: String) = requireActivity().setTitle(title)
}